/** @file
  Internal include file for BaseCryptLibNull.

Copyright (c) 2010 - 2017, Intel Corporation. All rights reserved.<BR>
SPDX-License-Identifier: BSD-2-Clause-Patent

**/

#ifndef __INTERNAL_CRYPT_LIB_H__
#define __INTERNAL_CRYPT_LIB_H__

#include <Base.h>
#include <Library/BaseCryptLib.h>
#include <Library/DebugLib.h>

#endif
